/*
 * (C) Copyright IBM Corp. 1998,1999  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package com.ibm.xml.parsers;

import com.ibm.xml.framework.*;
import org.xml.sax.*;

/**
 *
 * @version Revision: 65 1.12 src/com/ibm/xml/parsers/SAXParser.java, parser, xml4j2, xml4j2_0_6 
 */
public class SAXParser extends XMLParser
{
    public SAXParser() {
        getParserState().useDefaultStringPool();
        getParserState().useDefaultAttrPool();
        getParserState().useDefaultEntityPool();
        getParserState().useDefaultElementDeclPool();
    }
    //
    //
    //
    private DTDHandler fDTDHandler = null;
    private DocumentHandler fDocumentHandler = null;
    /**
     *
     */
    public void setEntityResolver(EntityResolver resolver) {
        getEntityHandler().setEntityResolver(resolver);
    }
    /**
     *
     */
    public void setDTDHandler(DTDHandler handler) {
        fDTDHandler = handler;
        setDocumentTypeHandler(handler == null ? null : this);
    }
    /**
     *
     */
    public void setDocumentHandler(DocumentHandler handler) {
        fDocumentHandler = handler;
        setDocumentHandler(handler == null ? null : this);
    }

    //
    // XMLDocumentTypeHandler interface
    //

    /**
     *  This function will be called when a &lt;!DOCTYPE...&gt; declaration is
     *  encountered.
     */
    public void doctypeDecl(int rootElementNameIndex) throws Exception {}

    /**
     *  This function will be called at the start of the "internal subset" of a 
     *  doctype declaration.
     */
    public void startInternalSubset() throws Exception {}

    /**
     *  This function will be called at the end of the "internal subset" of a 
     *  doctype declaration.
     */
    public void endInternalSubset() throws Exception {}

    /**
     *  This function will be called at the start of the "external subset" of a 
     *  doctype declaration.
     */
    public void startExternalSubset(int publicIdStringIndex, int systemIdStringIndex) throws Exception {}
    
    /**
     *  This function will be called at the end of the "external subset" of a 
     *  doctype declaration.
     */
    public void endExternalSubset() throws Exception {}
 
    /**
     *
     */
    public void elementDecl(int elementIndex) throws Exception {
    }
    /**
     *
     */
    public void attlistDecl(int elementIndex, int attDefIndex) throws Exception {
    }
    /**
     *
     */
    public void internalEntityDecl(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void externalEntityDecl(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void unparsedEntityDecl(int entityIndex) throws Exception {
        String name = getParserState().getStringPool().toString(getParserState().getEntityPool().getEntityName(entityIndex));
        String publicId = getParserState().getStringPool().toString(getParserState().getEntityPool().getPublicId(entityIndex));
        String systemId = getParserState().getStringPool().toString(getParserState().getEntityPool().getSystemId(entityIndex));
        String notation = getParserState().getStringPool().toString(getParserState().getEntityPool().getNotationName(entityIndex));
        fDTDHandler.unparsedEntityDecl(name, publicId, systemId, notation);
    }
    /**
     *
     */
    public void notationDecl(int notationIndex) throws Exception {
        String name = getParserState().getStringPool().toString(getParserState().getEntityPool().getNotationName(notationIndex));
        String publicId = getParserState().getStringPool().toString(getParserState().getEntityPool().getPublicId(notationIndex));
        String systemId = getParserState().getStringPool().toString(getParserState().getEntityPool().getSystemId(notationIndex));
        fDTDHandler.notationDecl(name, publicId, systemId);
    }

    //
    // XMLDocumentHandler interface
    //

    /**
     *
     */
    public boolean sendCharDataAsCharArray() {
        return true;
    }

    /**
     *
     */
    public void startDocument(int versionIndex,
                              int encodingIndex,
                              int standAloneIndex) throws Exception {
        StringPool  strPool = getParserState().getStringPool();
        strPool.orphanString(versionIndex);
        strPool.orphanString(encodingIndex);
        strPool.orphanString(standAloneIndex);
        
        fDocumentHandler.setDocumentLocator(getLocator());
        fDocumentHandler.startDocument();
    }
    /**
     *
     */
    public void endDocument() throws Exception {
        fDocumentHandler.endDocument();
    }
    /**
     *
     */
    public void startElement(int elementNameIndex, int attrListIndex) throws Exception {
        fDocumentHandler.startElement(getParserState().getStringPool().toString(elementNameIndex),
                                      getParserState().getAttrPool().getAttributeList(attrListIndex));
        getParserState().getAttrPool().releaseAttrList(attrListIndex);
    }
    /**
     *
     */
    public void endElement(int elementNameIndex) throws Exception {
        fDocumentHandler.endElement(getParserState().getStringPool().toString(elementNameIndex));
    }
    /**
     *
     */
    public void startEntityReference(int entityIndex) throws Exception {
    }
    /**
     *
     */
    public void endEntityReference(int entityIndex) throws Exception {
    }
    /**
     * Not called
     */
    public void characters(int dataIndex, boolean cdataSection) throws Exception {
        String s = getParserState().getStringPool().orphanString(dataIndex);
        fDocumentHandler.characters(s.toCharArray(), 0, s.length());
    }
    /**
     *Not called
     */
    public void ignorableWhitespace(int dataIndex, boolean cdataSection) throws Exception {
        String s = getParserState().getStringPool().orphanString(dataIndex);
        fDocumentHandler.ignorableWhitespace(s.toCharArray(), 0, s.length());
    }
    /**
     *
     */
    public void processingInstruction(int targetIndex, int dataIndex) throws Exception {
        fDocumentHandler.processingInstruction(getParserState().getStringPool().orphanString(targetIndex),
                                               getParserState().getStringPool().orphanString(dataIndex));
    }
    /**
     *
     */
    public void comment(int dataIndex) throws Exception {
        getParserState().getStringPool().releaseString(dataIndex);
    }
    /**
     *
     */
    public void characters(char ch[], int start, int length, boolean cdataSection) throws Exception {
        fDocumentHandler.characters(ch, start, length);
    }
    /**
     *
     */
    public void ignorableWhitespace(char ch[], int start, int length, boolean cdataSection) throws Exception {
        fDocumentHandler.ignorableWhitespace(ch, start, length);
    }
}
